// From LOL shield sample sketch
// modified by ikkei for ikkeiplexing shield 2010.12.16
// SW5: up
// SW8: down
//         2013-10-26 - update class library

#include <MsTimer2.h>
#include <ikkeiplexing.h>
//**************************************************************//
//  Name    : Pong for Arduino / Charlieplexing                 //
//  Author  : Benjamin Sonntag http://benjamin.sonntag.fr/      //
//  Modified: Matt Mets http://cibomahto.com                    //
//  Date    : 28 May 2010                                       //
//  Version : 0.2                                               //
//  Notes   : Uses Charlieplexing library to light up           //
//          : a matrix of 126 LEDs in a 9x14 grid               //
//          : from Jimmie P Rodgers www.jimmieprodgers.com      //
//**************************************************************//

ikkeiplexing iLedSign;

#define X_NUM 16
#define Y_NUM 8
#define X_MAX 15
#define Y_MAX 7

/** switch bit patteren */
const uint8_t SW1 = 0x80;
const uint8_t SW2 = 0x40;
const uint8_t SW3 = 0x20;
const uint8_t SW4 = 0x10;
const uint8_t SW5 = 0x08;
const uint8_t SW6 = 0x04;
const uint8_t SW7 = 0x02;
const uint8_t SW8 = 0x01;

#define SW_UP    SW5
#define SW_DOWN  SW8
//#define SW_UP    SW1 // for Tichaino uni
//#define SW_DOWN  SW2

/* ---------------------------------------------------------------------------*/
/** The figures from 0 to 9 encoded in 7 lines of 5 bits :
 */
uint16_t figures[][7] = { 
  {B01110,
   B10001,
   B10001,
   B10001,
   B10001,
   B10001,
   B01110},
  {B00100,
   B00110,
   B00100,
   B00100,
   B00100,
   B00100,
   B01110},
  {B01110,
   B10001,
   B10000,
   B01110,
   B00001,
   B00001,
   B11111},
  {B01110,
   B10001,
   B10000,
   B01110,
   B10000,
   B10001,
   B01110},
  {B01000,
   B01100,
   B01010,
   B01001,
   B11111,
   B01000,
   B01000},
  {B11111,
   B00001,
   B00001,
   B01111,
   B10000,
   B10000,
   B01111},
  {B01110,
   B10001,
   B00001,
   B01111,
   B10001,
   B10001,
   B01110},
  {B11111,
   B10000,
   B01000,
   B01000,
   B00100,
   B00100,
   B00100},
  {B01110,
   B10001,
   B10001,
   B01110,
   B10001,
   B10001,
   B01110},
  {B01110,
   B10001,
   B10001,
   B11110,
   B10000,
   B10000,
   B01111}
};

  
int x,y,dx,dy;
int sh1y,sh2y,s1,s2;
//uint8_t keydata;

/* ---------------------------------------------------------------------------*/
/* Arduino setup func
 */
void setup() {
//  iLedSign.Init(DOUBLE_BUFFER);
  iLedSign.Init(IKKEIPLEXING, SINGLE_BUFFER);
  
  x = 3;
  y = 7;
  sh1y=3;
  sh2y=3;
  dx = 1;
  dy = 1;
  s1 = 0;
  s2 = 0;
  
  randomSeed(analogRead(0));
  drawscores();
}


/* ---------------------------------------------------------------------------*/
/* Arduino main loop 
 */
void loop() {
  int ct1,ct2,randommove;
  static int sh2y = 4;
  // The Ball shall bounce on the walls : 
  if (x==(X_NUM-2) || x==1) {
    dx=-dx;
    // Collision detection
    if (x==1) {
      // check the first ship (left side)
      if (sh1y!=y && sh1y+1!=y) {
        s2++;
        drawscores();
        checkscores();
      }
    } else {
      // check the second ship (right side)
      if (sh2y!=y && sh2y+1!=y) {
        s1++;
        drawscores();
        checkscores();
      }
    }
  }
  if (y==Y_MAX || y==0) dy=-dy;
 
  // Clear the non-active screen
  iLedSign.Clear();

  // Move the BALL : 
  x=x+dx; 
  y=y+dy;
  
  // Draw the ball :
  iLedSign.Set(x,y,1);

  // Draw the Ship
  iLedSign.Set(0, sh1y, 1);
  iLedSign.Set(0, sh1y+1, 1);
  iLedSign.Set(X_MAX, sh2y, 1);
  iLedSign.Set(X_MAX, sh2y+1, 1);
  
  // The ships moves when the ball go in their direction. They follow it magically ;) :   
  /* This code is too smart, in fact he is perfekt :) 
  if (dx<0) {
    if (sh1y>y) {
      sh1y--;
    }
    if (sh1y<y) {
      sh1y++;
    }
  }
  // let's code a dummy one 
  */
  
  if (dx>0) {
    // the ball goes away from me, let's move randomly
    randommove=random(0,3);
    if (randommove==0) {
      sh1y--;
    }
    if (randommove==1) {
      sh1y++;
    }
  } else {
    if (sh1y>y && (random(0,12)<10 || x<3)) {
      sh1y--;
    }
    if (sh1y<y && (random(0,12)<10 || x<3)) {
      sh1y++;
    }
    if (random(0,8)==0) {
      if (sh1y>y) {
        sh1y++;
      }
      if (sh1y<y) {
        sh1y--;
      }      
    }
  }
  

  // Human Player 
  // 1/4 of the variator is used. If we use it fully, it's too hard to play.
  // To use it fully replace 36 by 146
  // sh2y=analogRead(5)/72; 
  byte keydata = iLedSign.GetSW(SW_LEVEL); // get switch edge
  if ((keydata & SW_UP) != 0){
    if (sh2y >=0){
      sh2y--;
    }
  } else if ((keydata & SW_DOWN) != 0){
    if (sh2y < Y_MAX){
      sh2y++;
    }
  }
  
  // Sanity checks for the ships : 
  if (sh1y>(Y_MAX-1)) sh1y=Y_MAX-1;
  if (sh2y>(Y_MAX-1)) sh2y=Y_MAX-1;
  if (sh1y<0) sh1y=0;
  if (sh2y<0) sh2y=0;

  // swap the screens ;) (sometime we may need this double-buffer algorithm...
  // of course, as of today it's a little bit overkill ...)

  iLedSign.Flip();

  // Display the bitmap some times 
  delay(200);
  
  // loop :) 
}


/* ---------------------------------------------------------------------------*/
/** Check if a player won !
 * If one of the players won, let's show a funny animation ;) 
 */
void checkscores() {  
  // TODO : DO the animation ;)
}
  
  
/* ---------------------------------------------------------------------------*/
/** Draw the scores in a lovely scrolling :) 
 * Use the current active screen brutally ...
 */
void drawscores() {
  int i,j,ps1,ps2,ct2;
  
  for(ps1=0;ps1<8;ps1++) {
    iLedSign.Clear();  // Clear the active screen

    iLedSign.Set(7,4,1); // dash between the scores
    iLedSign.Set(8,4,1);
    
    // Fill it with both scores : 
    // Left score goes up>down
    i=ps1; j=6;
    while (i>=0 && j>=0) {
      for(uint8_t k = 0; k < 5; k++) {
        iLedSign.Set(k+1,i,(figures[s1][j] >> k) & 1 );
      }
      
      i--; j--;
    }
    // Right score goes down>up
    ps2=8-ps1;
    i=ps2; j=0;
    while (i<=Y_MAX && j<=6) {
      for(uint8_t k = 0; k < 5; k++) {
        iLedSign.Set(k+10,i,(figures[s2][j] >> k) & 1 );
      }

      i++; j++;
    }    

    iLedSign.Flip();
    delay(200);
  }
  
  delay(1500);
  
  iLedSign.Clear(0);

  if (s1==9 || s2==9) {
    for(ps1=0;ps1<3;ps1++) {
      iLedSign.Flip();      
      delay(300);
      
      iLedSign.Flip();
      delay(600);
      
    }
    delay(1500);

    s1=0; s2=0;
    drawscores();
  }
}


